#ifndef DS50FragmentGenerator_hh
#define DS50FragmentGenerator_hh

#include <condition_variable>
#include <vector>

#include "fhiclcpp/fwd.h"
#include "artdaq/Application/CommandableFragmentGenerator.hh"

namespace ds50 {
  class DS50FragmentGenerator : public artdaq::CommandableFragmentGenerator {
    public:
      DS50FragmentGenerator(const fhicl::ParameterSet &);
      DS50FragmentGenerator(const DS50FragmentGenerator &) = delete;
      virtual ~DS50FragmentGenerator() noexcept = default;

    void perfreset ();

    virtual void start () override ;
    virtual void pause () override ;
    virtual void resume () override ;
    virtual void stop () override ;
    virtual std::string report () override ; 

  private:
      virtual void start_ () {}
      virtual void pause_ () {}
      virtual void resume_ () {}
      virtual void stop_ () {}
      virtual std::string report_ () { return ""; } 
      virtual void perfreset_ () {}

      virtual bool getNext_ (artdaq::FragmentPtrs & output) final;
      virtual bool getNext__ (artdaq::FragmentPtrs & output) = 0;

      int sleep_us_;
      bool init_only_;
      std::condition_variable stop_wait_;

      struct stats {
	int run_number;
	int call_count;
	double avg_frag_size;
	int read_count;
      } stats_;
     
  };
}

#endif /* DS50FragmentGenerator_hh */
