////////////////////////////////////////////////////////////////////////
// Class:       MonitorBaseline
// Module Type: analyzer
// File:        MonitorBaseline_module.cc
//
// Generated at Fri Mar  1 13:10:14 2013 by Nicola Rossi using artmod
// from art v1_03_03.
////////////////////////////////////////////////////////////////////////


#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Run.h"
#include "art/Utilities/InputTag.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V172xFragment.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TH2F.h"
#include "TStyle.h"

#include <numeric>
#include <cmath>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>

namespace ds50 {
  class MonitorBaseline : public art::EDAnalyzer {
    public:
      explicit MonitorBaseline(fhicl::ParameterSet const & p);
      virtual ~MonitorBaseline();

      void analyze(art::Event const & e) override;
      void beginRun(art::Run const &) override;

    private:
      std::string board_type_;
      std::unique_ptr<TCanvas> canvas_;
      std::unique_ptr<TH2F> hist_[2];
      bool random_only_;
      int prescale_;

      art::RunNumber_t current_run_;
  };
}

ds50::MonitorBaseline::MonitorBaseline(fhicl::ParameterSet const & ps): art::EDAnalyzer(ps), board_type_(ps.get<std::string> ("board_type", "V1720")), random_only_(ps.get<bool> ("random_only", false)), prescale_(ps.get<int> ("prescale", 50)), current_run_(0) { }

ds50::MonitorBaseline::~MonitorBaseline() { }

void ds50::MonitorBaseline::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) return;

  if(v1495->size() != 1) {
    mf::LogError ("MonitorTrigger") << v1495->size() << " fragment for V1495";
    return;
  }
  const auto& t_frag((*v1495)[0]);
  if (random_only_ && V1495Fragment(t_frag).trigger_type () != V1495Fragment::random) return;

  art::Handle<artdaq::Fragments> v172x;
  e.getByLabel ("daq", board_type_, v172x);
  if (!v172x.isValid ())  {
    e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
    if (!v172x.isValid ()) return;
  }

  for (size_t i = 0; i < v172x->size(); ++i) {
    const auto& frag((*v172x)[i]);

    V172xFragment b(frag);
    size_t board_id = b.board_id ();

    size_t n_samples = b.adc_values_for_channel ();
    if (V1495Fragment(t_frag).trigger_type () != V1495Fragment::random) {
      if (!frag.hasMetadata () || !t_frag.hasMetadata ()) continue;
      float g2trigger_delay_us = (t_frag.metadata<V1495Fragment::metadata> ()->inhibit_20ns) ? (t_frag.metadata<V1495Fragment::metadata> ()->g2_gate_20ns / 50 + 0.85): 0; // inhibit was 0 before new metadata
      n_samples = n_samples * (100. - frag.metadata<V172xFragment::metadata> ()->post_trigger) / 100 - frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz * (0.5 + g2trigger_delay_us);
      if (n_samples <= 0) continue;
    }

    for (int ch = 0; ch < 8; ++ch) {
      int lg = board_id * 8 + ch; 
      if (!b.channel_present (ch)) continue;

      // mean = <x>
      double m_temp = std::accumulate (b.chDataBegin (ch), b.chDataBegin (ch) + n_samples, 0.) / n_samples;
      hist_[0] -> Fill(lg, m_temp);

      // sigma = <x^2> - <x>^2
      double s_temp = std::sqrt (std::inner_product(b.chDataBegin (ch), b.chDataBegin (ch) + n_samples, b.chDataBegin (ch), 0.) / n_samples - m_temp * m_temp);
      hist_[1] -> Fill(lg, s_temp);
    }

    if (!(frag.sequenceID () % prescale_)) {
      canvas_ -> cd(1)->Modified ();
      canvas_ -> cd(2)->Modified ();
      canvas_ -> Update();
    }
  }
}

void ds50::MonitorBaseline::beginRun(art::Run const &e) {
  if (e.run () == current_run_) return;
  current_run_ = e.run ();

  canvas_ = 0;
  hist_[0] = 0;
  hist_[1] = 0;

  canvas_ = std::unique_ptr<TCanvas>(new TCanvas("Baseline"));
  ((TRootCanvas*)canvas_->GetCanvasImp ())->DontCallClose ();
  canvas_ -> Divide(2, 1);  

  canvas_ -> cd(1);
  hist_[0] = std::unique_ptr<TH2F> (new TH2F("mean","Mean",40,0,40,4096,0,4096));
  hist_[0] -> Draw("COL"); // or "BOX"
  hist_[0] -> SetStats(0);

  canvas_ -> cd(2);
  hist_[1] = std::unique_ptr<TH2F> (new TH2F("sigma","Stdev",40,0,40,1024,0,3));
  hist_[1] -> Draw("COL"); 
  hist_[1] -> SetStats(0);

  canvas_->Update();
}


DEFINE_ART_MODULE(ds50::MonitorBaseline)
