////////////////////////////////////////////////////////////////////////
// Class:       MonitorSpectrum
// Module Type: analyzer
// File:        MonitorSpectrum_module.cc
//
// Generated at Sun Dec  2 12:23:06 2012 by Alessandro Razeto & Nicola Rossi using artmod
// from art v1_02_04.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Run.h"
#include "art/Utilities/InputTag.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V172xFragment.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"
#include "ds50daq/online/ArtServices/DBInterface.hh"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TH1F.h"
#include "TAxis.h"

#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>

namespace ds50 {
  class MonitorSpectrum : public art::EDAnalyzer {
    public:
      explicit MonitorSpectrum (fhicl::ParameterSet const & p);
      virtual ~MonitorSpectrum ();

      void analyze (art::Event const & e) override;
      void beginRun(art::Run const &) override;

    private:
      std::string board_type_;
      std::unique_ptr<TCanvas> canvas_;
      std::unique_ptr<TH1F> spectrum_, prescaled_spectrum_;
      std::unique_ptr<TH1F> trigger_digital_sum_, prescaled_trigger_digital_sum_;
      std::vector<bool> normals_;
      int prescale_;
      float start_us_, stop_us_;
      std::vector<float> range_;

      art::RunNumber_t current_run_;
  };
}

ds50::MonitorSpectrum::MonitorSpectrum (fhicl::ParameterSet const & ps): art::EDAnalyzer(ps), board_type_(ps.get<std::string> ("board_type", "V1720")), prescale_(ps.get<int> ("prescale", 50)), start_us_(ps.get<float> ("start_us")), stop_us_(ps.get<float> ("stop_us")), range_(ps.get<std::vector<float>> ("range", std::vector<float>(0))), current_run_(0) {
}

ds50::MonitorSpectrum::~MonitorSpectrum() { }

void ds50::MonitorSpectrum::analyze (art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495; 
  e.getByLabel ("daq", "V1495", v1495); 
  if (!v1495.isValid ()) return;

  if(v1495->size() != 1) {
    mf::LogError ("MonitorSer") << v1495->size() << " fragment for V1495";
    return;
  }
  V1495Fragment::trigger_types trigger_type = V1495Fragment((*v1495)[0]).trigger_type ();
  if (trigger_type != V1495Fragment::tpc && trigger_type != V1495Fragment::tpc_low && trigger_type != V1495Fragment::tpc_high) return;

  int digital_sum = V1495Fragment((*v1495)[0]).tpc_digital_sum ();
  if (trigger_type == V1495Fragment::tpc_high) prescaled_trigger_digital_sum_->Fill (digital_sum);
  else trigger_digital_sum_->Fill (digital_sum);

  art::Handle<artdaq::Fragments> v172x;
  e.getByLabel ("daq", board_type_, v172x);
  if (!v172x.isValid ())  {
    e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
    if (!v172x.isValid ()) return;
  }

  float total = 0;
  for (size_t i = 0; i < v172x->size(); ++i) {
    const auto& frag((*v172x)[i]);
    V172xFragment b(frag);
    size_t board_id = b.board_id ();

    if (!frag.hasMetadata ()) continue;
    int pre_samples = b.adc_values_for_channel () * (100 - frag.metadata<V172xFragment::metadata> ()->post_trigger) / 100;
    int start = int(start_us_ * frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz) + pre_samples;
    int stop = int(stop_us_ * frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz) + pre_samples;

    for (int ch=0; ch<8; ++ch) {
      int lg = board_id * 8 + ch;                        
      if (!b.channel_present (ch) || lg >= 40) continue;

      if (normals_[lg]) {
	//double baseline = (std::accumulate (b.chDataBegin (ch) + (2 * start - stop), b.chDataBegin (ch) + start, 0.) + std::accumulate (b.chDataBegin (ch) + stop, b.chDataBegin (ch) + (2 * stop - start), 0.)) / 2;
	double baseline = std::accumulate (b.chDataBegin (ch) + std::max((2 * start - stop), 0), b.chDataBegin (ch) + start, 0.);
	double sum = std::accumulate (b.chDataBegin (ch) + start, b.chDataBegin (ch) + stop, 0.) - baseline;
	total += sum * -1;
      }
    }
  }

  if (trigger_type == V1495Fragment::tpc_high) prescaled_spectrum_->Fill (total);
  else spectrum_->Fill (total);

  if (int(spectrum_->GetEntries ()) % prescale_) {
    canvas_->cd (1)-> Modified();
    canvas_->cd (2)-> Modified();
    canvas_->Update ();
  }
}

void ds50::MonitorSpectrum::beginRun(art::Run const &e) { 
  if (e.run () == current_run_) return;
  current_run_ = e.run ();

  canvas_ = 0;
  spectrum_ = 0;
  prescaled_spectrum_ = 0;

  canvas_ = std::unique_ptr<TCanvas>(new TCanvas("Spectrum"));
  ((TRootCanvas*)canvas_->GetCanvasImp ())->DontCallClose ();
  canvas_->Divide(1,2);

  canvas_->cd (1);
  if (!range_.size ()) range_ = { -100, 5000, 500 };
  else if (range_.size () == 2) range_.push_back (500);
  spectrum_ = std::unique_ptr<TH1F>(new TH1F("simple_spectrum", "Simple spectrum", range_[2], range_[0], range_[1]));
  spectrum_->SetLineColor (1);
  spectrum_->Draw ();
  prescaled_spectrum_ = std::unique_ptr<TH1F>(new TH1F("prescaled_simple_spectrum", "Prescaled simple spectrum", range_[2], range_[0], range_[1]));
  prescaled_spectrum_->SetLineColor (2);
  prescaled_spectrum_->Draw ("same");

  canvas_->cd (2);
  trigger_digital_sum_ = std::unique_ptr<TH1F>(new TH1F("simple_trigger_digital_sum", "Trigger digital sum", 250, 0, 2500));
  trigger_digital_sum_->SetLineColor (1);
  trigger_digital_sum_->Draw ();
  prescaled_trigger_digital_sum_ = std::unique_ptr<TH1F>(new TH1F("prescaled_trigger_digital_sum", "Prescaled trigger digital sum", 250, 0, 2500));
  prescaled_trigger_digital_sum_->SetLineColor (2);
  prescaled_trigger_digital_sum_->Draw ("same");

  try {
    art::ServiceHandle<ds50::DBInterface> dbi;
    ds50::db::result r = dbi->latest ("calib.pmt_geometry", "");
    for (size_t i = 0; i < r.cell_elements("r", 0); i++) normals_.push_back (r.get<float>("r", 0, i) >= 0);
  } catch (const cet::exception& e) {
    mf::LogError ("MonitorSpectrum") << "cannot read db, proceeding with default: " << e.what ();
    normals_.resize (40, true);
    normals_[38] = normals_[39] = false;
  }
}


DEFINE_ART_MODULE(ds50::MonitorSpectrum)
