////////////////////////////////////////////////////////////////////////
// Class:       RunInfo
// Module Type: analyzer
// File:        RunInfo_module.cc
//
// Generated at Thu Mar 13 09:24:45 UTC 2014 from Alessandro Razeto
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"
#include "ds50daq/DAQ/V172xFragment.hh"
#include "ds50daq/DAQ/CompressedV172x.hh"
#include "ds50daq/online/ArtServices/DBInterface.hh"

#include <iostream>
#include <iomanip>

namespace ds50 {
  class RunInfo : public art::EDAnalyzer {
    public:
      explicit RunInfo(fhicl::ParameterSet const & p);
      virtual ~RunInfo();

      void beginRun(art::Run const &) override;
      void analyze(art::Event const & e) override;
      void endRun(art::Run const &) override;

    private:
      time_t start_time_, stop_time_;
      suseconds_t start_time_us_, stop_time_us_;
      double live_time_v1495_, dead_time_v1495_;
      V1495Fragment::count_t gps_coarse_s_, gps_fine_20ns_, gps_pps_20ns_;
      long events_, invalid_, overflow_;
      std::string board_type_, compression_type_, metadata_, type_;
  };
}

ds50::RunInfo::RunInfo(fhicl::ParameterSet const &p): art::EDAnalyzer(p), board_type_(p.get<std::string> ("board_type", "V1720")), compression_type_(p.get<std::string> ("compression_type", "huffdiff")) { }

ds50::RunInfo::~RunInfo() { }

void ds50::RunInfo::beginRun(art::Run const &) {
  start_time_ = 0;
  stop_time_ = 0;
  live_time_v1495_ = 0;
  dead_time_v1495_ = 0;
  events_ = 0;
  invalid_ = 0;
  overflow_ = 0;
  gps_coarse_s_ = 0;
  metadata_ = "";
  type_ = art::ServiceHandle<ds50::DBInterface>()->current ("run.daqruns_old_format", "").get<std::string> ("type", 0);
}

void ds50::RunInfo::analyze(art::Event const & e) {
  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid () || v1495->size() != 1 || !((*v1495)[0]).hasMetadata ()) {
    invalid_++;
    return;
  }

  events_++;

  const auto& t_frag((*v1495)[0]);
  V1495Fragment b(t_frag);
  V1495Fragment::metadata const* md = t_frag.metadata<V1495Fragment::metadata> ();

  time_t trigger_time =  md->trigger_time.tv_sec;
  suseconds_t trigger_time_us = md->trigger_time.tv_usec;
  if (!start_time_) { 
    start_time_ = trigger_time;
    start_time_us_ = trigger_time_us;
  }

  if (trigger_time < stop_time_) mf::LogError ("RunInfo") << "time is flowing back " << difftime (trigger_time, stop_time_) << " s at event " << b.trigger_counter ();
  else {
    stop_time_ = trigger_time;
    stop_time_us_ = trigger_time_us;
  }

  if ((b.live_time_100ns () == 0xFFFFFFFF) || (b.incremental_inhibit_time_100ns () == 0xFFFFFFFF)) overflow_++;

  live_time_v1495_ += b.live_time_100ns () * 100e-9;
  dead_time_v1495_ += b.incremental_inhibit_time_100ns () * 100e-9;

  if (gps_coarse_s_ <= b.gps_coarse_s ()) {
    gps_coarse_s_ = b.gps_coarse_s ();
    gps_fine_20ns_ = b.gps_fine_20ns ();
    gps_pps_20ns_ = b.gps_pps_20ns ();
  }

  if (!metadata_.size ()) {
    V172xFragment::metadata const* dmd = 0;

    art::Handle<artdaq::Fragments> v172x;
    e.getByLabel ("daq", board_type_, v172x);
    if (!v172x.isValid ())  {
      e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
      if (!v172x.isValid ()) { // Use compressed data
	art::Handle<CompressedV172x> v172x_compressed;
	e.getByLabel(compression_type_ + board_type_, board_type_, v172x_compressed);
	if (!v172x_compressed.isValid ()) return;
	const auto& d_frag = v172x_compressed->headerOnlyFrag (0);
	if (!d_frag.hasMetadata ()) return;
	dmd = d_frag.metadata<V172xFragment::metadata> ();
      }
    }
    if (!dmd) { // if 0 we have uncompressed data
      const auto& d_frag((*v172x)[0]);
      if (!d_frag.hasMetadata ()) return;
      dmd = d_frag.metadata<V172xFragment::metadata> ();
    }

    std::ostringstream str;
    str << "Trigger FW: " << std::hex << std::showbase << int(md->firmware_version) << std::dec << std::endl;
    str << "Trigger Threshold: " << int(md->threshold) << std::endl;
    if (e.run () > 7000 && md->inhibit_20ns > 0) {
      str << "Trigger Inhibit: " << md->inhibit_20ns / 50 << " us" << std::endl;
      str << "Trigger G2 Threshold: " << md->g2_threshold << std::endl;
      str << "Trigger G2 Gate: " << int(md->g2_gate_20ns / 50) << " us" << std::endl;
      str << "Trigger G2 Prescale " << int(md->g2_prescale) << std::endl;
    }
    str << "Digitizer Gate: " << dmd->record_length / dmd->sample_rate_MHz << " us" << std::endl;
    str << "Digitizer Pretrigger: " << dmd->record_length * (100 - dmd->post_trigger) / 100 / dmd->sample_rate_MHz << " us" <<  std::endl;
    metadata_ = str.str ();
  }
}


void ds50::RunInfo::endRun(art::Run const &e) {
  std::ostringstream start_s, stop_s;
  start_s << start_time_ << "." << std::setw(6) << std::setfill('0') << start_time_us_;
  stop_s << stop_time_ << "." << std::setw(6) << std::setfill('0') << stop_time_us_;
  double dt = difftime (stop_time_, start_time_) + (stop_time_us_ - start_time_us_) * 1e-6;
  std::cout << "Run: " << e.run () << std::endl;
  std::cout << "Type: " << type_ << std::endl;
  std::cout << "Events: " << events_ << std::endl;
  std::cout << "Invalid: " << invalid_ << std::endl;
  std::cout << "Overflow: " << overflow_ << std::endl;
  std::cout << "Start: " << start_s.str () << std::endl;
  std::cout << "Stop: " << stop_s.str () << std::endl;
  std::cout << "Runtime DAQ: " << dt << std::endl;
  std::cout << "Runtime GPS: " << double(gps_coarse_s_) + gps_fine_20ns_ * 20e-9 << std::endl;
  std::cout << "Livetime v1495: " << live_time_v1495_ << std::endl;
  std::cout << "Deadtime v1495: " << dead_time_v1495_ << std::endl;
  std::cout << "Missing time: " << dt - live_time_v1495_ - dead_time_v1495_ << std::endl;
  std::cout << metadata_ << std::endl;

  try {
    art::ServiceHandle<ds50::DBInterface> dbi;
    ds50::db::result r = dbi->latest ("run.tpc_info", "");
    r.wipe ();
    r.set ("version", 0, "RunInfo");
    r.set ("run", 0, e.run ());
    r.set ("events", 0, events_);
    r.set ("invalid_count", 0, invalid_);
    r.set ("start", 0, dbi->query ("select to_timestamp(" + start_s.str () + ")").get<std::string> (0,0));
    r.set ("stop", 0, dbi->query ("select to_timestamp(" + stop_s.str () + ")").get<std::string> (0,0));
    r.set ("runtime_daq", 0, dt);
    r.set ("runtime_gps", 0, double(gps_coarse_s_) + gps_fine_20ns_ * 20e-9);
    r.set ("livetime_v1495", 0, live_time_v1495_);
    r.set ("deadtime_v1495", 0, dead_time_v1495_);
    r.set ("metadata", 0, metadata_);
    r.set ("type", 0, type_);

    dbi->insert (r, "RunInfo");
  } catch (const cet::exception& e) {
    mf::LogError ("RunInfo") << "exception: " << e.what ();
  }
}

DEFINE_ART_MODULE(ds50::RunInfo)
