////////////////////////////////////////////////////////////////////////
// Class:       RunMetadata
// Module Type: analyzer
// File:        RunMetadata_module.cc
//
// Generated at Sat Feb 22 14:32:16 2014 by Alessandro Razeto using artmod
// from cetpkgsupport v1_05_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"
#include "ds50daq/DAQ/V172xFragment.hh"
#include <iostream>
#include <iomanip>
#include <ctime>


namespace ds50 {
  class RunMetadata : public art::EDAnalyzer {
    public:
      explicit RunMetadata(fhicl::ParameterSet const & p);
      virtual ~RunMetadata();

      void analyze(art::Event const & e) override;
      void beginRun(art::Run const &) override;
      void endRun(art::Run const &) override;

    private:
      std::string board_type_;
      bool first_;
  };
}


ds50::RunMetadata::RunMetadata(fhicl::ParameterSet const & p): art::EDAnalyzer(p), board_type_(p.get<std::string> ("board_type", "V1720")), first_(false) {}

ds50::RunMetadata::~RunMetadata() {}

void ds50::RunMetadata::beginRun(art::Run const &e) {
  first_ = false;

  std::cerr << "Metatdata for run " << e.run () << std::endl;
}


void ds50::RunMetadata::analyze(art::Event const & e) {
  if (first_) return;

  art::Handle<artdaq::Fragments> v1495;
  e.getByLabel ("daq", "V1495", v1495);
  if (!v1495.isValid ()) return;
  const auto& t_frag((*v1495)[0]);
  if (!t_frag.hasMetadata ()) return;

  std::cerr << "  Trigger FW: " << std::hex << std::showbase << int(t_frag.metadata<V1495Fragment::metadata> ()->firmware_version) << std::dec << std::endl;
  std::cerr << "  Trigger Threshold: " << int(t_frag.metadata<V1495Fragment::metadata> ()->threshold) << std::endl;
  std::cerr << "  Trigger Inhibit: " << t_frag.metadata<V1495Fragment::metadata> ()->inhibit_20ns / 50 << " us" << std::endl;
  std::cerr << "  Trigger G2: threshold " << t_frag.metadata<V1495Fragment::metadata> ()->g2_threshold << ", gate " << int(t_frag.metadata<V1495Fragment::metadata> ()->g2_gate_20ns / 50) << " us, prescale " << int(t_frag.metadata<V1495Fragment::metadata> ()->g2_prescale) << std::endl;

  art::Handle<artdaq::Fragments> v172x;
  e.getByLabel ("daq", board_type_, v172x);
  if (!v172x.isValid ())  {
    e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
    if (!v172x.isValid ()) return;
  }
  const auto& d_frag((*v172x)[0]);
  if (!d_frag.hasMetadata ()) return;

  std::cerr << "  Digitizer gate: " << d_frag.metadata<V172xFragment::metadata> ()->record_length / d_frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz << " us" << std::endl;
  std::cerr << "  Digitizer pretrigger: " << d_frag.metadata<V172xFragment::metadata> ()->record_length * (100 - d_frag.metadata<V172xFragment::metadata> ()->post_trigger) / 100 / d_frag.metadata<V172xFragment::metadata> ()->sample_rate_MHz << " us" <<  std::endl;

  first_ = true;
}

void ds50::RunMetadata::endRun(art::Run const &e) {
  time_t start_time = e.beginTime ().value (), end_time = e.endTime ().value ();
  std::cerr << "  Number of events: " << "-" << std::endl;
  std::cerr << "  Start time: " <<  std::asctime (std::gmtime (&start_time)); // put_time is missing
  std::cerr << "  End time: " << std::asctime (std::gmtime (&end_time));
}

DEFINE_ART_MODULE(ds50::RunMetadata)
